/*****************************************************************************
 *            Created: 6/05/2012
 *          Copyright: (C) Copyright David Goodwin, 2012
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#ifndef LIFEWIDGET_H
#define LIFEWIDGET_H

#include <QWidget>
#include <QPixmap>
#include <QMutex>

#include "grid.h"

class LifeWidget : public QWidget
{
    Q_OBJECT
public:
    LifeWidget(QWidget *parent = 0);
    ~LifeWidget();
    
    void init();

    unsigned long getGridHeight() { return grid_height; }
    unsigned long getGridWidth() { return grid_width; }
    
    void saveGrid(QString filename);
    void saveTemplate(QString filename, QString description);
    void loadGrid(QString filename);


public slots:
    void async_iterate();
    void iterate();
    void randomize();
    void reset();
    void crop();
    void placeTemplate(QString filename);
    void clearTemplate();

protected:
    virtual void paintEvent(QPaintEvent *);
    virtual void resizeEvent(QResizeEvent *);
    virtual void mouseMoveEvent(QMouseEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void enterEvent(QEvent *);
    virtual void leaveEvent(QEvent *);

private:
    QMutex grid_mutex;
    grid_t grid;
    grid_t secondary_grid;
    uint32_t grid_width;
    uint32_t grid_height;

    bool mouse_is_in_widget;

    int hover_x, hover_y;
    int old_hover_x, old_hover_y;

    QPixmap pix;
    QMutex pix_mutex;

    /* Template placement stuff */
    QPixmap template_pix;
    grid_t template_grid;
    uint32_t template_width;
    uint32_t template_height;
    bool template_mode;

    unsigned int count_neighbours(unsigned int x, unsigned int y);

    void redraw_pixmap(QPixmap *pixmap, grid_t grid,
                       uint32_t width, uint32_t height,
                       bool is_template = false);
    void redraw_pixmap();
};

#endif // LIFEWIDGET_H
