/*****************************************************************************
 *            Created: 6/05/2012
 *          Copyright: (C) Copyright David Goodwin, 2012
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#include <QFileDialog>
#include <QCoreApplication>
#include <QClipboard>

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "aboutdialog.h"
#include "createtemplatedialog.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    updateTimerInterval(50);
    timer.setSingleShot(false);

    // File
    connect(ui->actionNew_Random, SIGNAL(triggered()), ui->widget, SLOT(randomize()));
    connect(ui->actionReset, SIGNAL(triggered()), ui->widget, SLOT(reset()));
    connect(ui->actionLoad, SIGNAL(triggered()), this, SLOT(loadGrid()));
    connect(ui->actionSave, SIGNAL(triggered()), this, SLOT(saveGrid()));
    connect(ui->actionSave_As_Template, SIGNAL(triggered()), this, SLOT(saveTemplate()));
    //-
    connect(ui->actionE_xit, SIGNAL(triggered()), this, SLOT(exit()));

    // Edit
    connect(ui->actionPlace_Block, SIGNAL(triggered()), this, SLOT(singleBlockTool()));
    connect(ui->actionPlace_template, SIGNAL(triggered()), this, SLOT(templateTool()));
    // -
    connect(ui->action_Open_Template_File, SIGNAL(triggered()),
            this, SLOT(loadTemplateFile()));
    //conenct(ui->actionPaste_Template, SIGNAL(triggered()), this, SLOT(pasteTemplate()));
    // -
    connect(ui->actionCrop, SIGNAL(triggered()), ui->widget, SLOT(crop()));

    // View
    connect(ui->action_Template_Browser, SIGNAL(triggered()),
            this, SLOT(loadTemplate()));

    // Simulation
    connect(ui->actionStep, SIGNAL(triggered()), ui->widget, SLOT(async_iterate()));
    connect(ui->actionRun, SIGNAL(triggered()), this, SLOT(run()));
    connect(ui->actionStop, SIGNAL(triggered()), this, SLOT(stop()));
    //-
    connect(ui->actionIncreaseSpeed, SIGNAL(triggered()), this, SLOT(speedUp()));
    connect(ui->actionReduceSpeed, SIGNAL(triggered()), this, SLOT(slowDown()));

    // Help
    connect(ui->action_About, SIGNAL(triggered()), this, SLOT(showAbout()));

    // Misc
    connect(&timer, SIGNAL(timeout()), ui->widget, SLOT(async_iterate()));
    connect(&tb, SIGNAL(templateSelected(QString)),
            ui->widget, SLOT(placeTemplate(QString)));
    connect(&tb, SIGNAL(templateSelected(QString)),
            this, SLOT(templateSelected(QString)));



}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::closeEvent(QCloseEvent *event) {
    exit();
}

void MainWindow::exit() {
    QCoreApplication::exit();
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::saveGrid() {
    QString filename = QFileDialog::getSaveFileName(this,"Save Grid","","Grid files (*.zgf)");
    ui->widget->saveGrid(filename);
}

void MainWindow::saveTemplate() {
    CreateTemplateDialog ctd;
    ctd.exec();

    QString description = ctd.getDescription();
    QString filename = ctd.getFilename();

    ui->widget->saveTemplate(filename, description);
}

void MainWindow::loadGrid() {
    QString filename = QFileDialog::getOpenFileName(this, "Load Grid","","Grid files (*.zgf)");
    ui->widget->loadGrid(filename);
}

void MainWindow::speedUp() {
    if (timer.interval() >= 20) {
        updateTimerInterval(timer.interval() - 10);
        if (timer.interval() == 10)
            ui->actionIncreaseSpeed->setEnabled(false);
    }
}

void MainWindow::slowDown() {
    updateTimerInterval(timer.interval() + 10);
    if (timer.interval() > 10)
        ui->actionIncreaseSpeed->setEnabled(true);
}

void MainWindow::run() {
    timer.start();
    ui->actionRun->setChecked(true);
    ui->actionStop->setChecked(false);
}

void MainWindow::stop() {
    timer.stop();
    ui->actionRun->setChecked(false);
    ui->actionStop->setChecked(true);
}

void MainWindow::updateTimerInterval(int interval) {
    timer.setInterval(interval);
}

void MainWindow::showAbout() {
    AboutDialog ad;
    ad.exec();
}

void MainWindow::templateTool() {

    QString templateFile = tb.currentTemplate();

    if (!templateFile.isEmpty())
        ui->widget->placeTemplate(tb.currentTemplate());

    ui->actionPlace_Block->setChecked(false);
    ui->actionPlace_template->setChecked(true);

    if (!tb.isVisible())
        loadTemplate();
}

void MainWindow::templateSelected(QString) {
    ui->actionPlace_Block->setChecked(false);
    ui->actionPlace_template->setChecked(true);
}

void MainWindow::singleBlockTool() {
    ui->widget->clearTemplate();
    ui->actionPlace_Block->setChecked(true);
    ui->actionPlace_template->setChecked(false);
}

void MainWindow::loadTemplate() {
    if (tb.isVisible()) {
        ui->action_Template_Browser->setChecked(false);
        tb.hide();
    } else {
        tb.show();
        ui->action_Template_Browser->setChecked(true);
    }
}

void MainWindow::templateBrowserClosed() {
    ui->action_Template_Browser->setChecked(false);
}

void MainWindow::loadTemplateFile() {
    QString filename = QFileDialog::getOpenFileName(this,
                                                    "Open Template",
                                                    "",
                                                    "Grid template files (*.zgt)");
    if (!filename.isEmpty()) {
        if (!tb.isVisible())
            tb.show();
        tb.previewFile(filename);
    }
}

// Not used.
void MainWindow::pasteTemplate() {
    QClipboard *clipboard = QApplication::clipboard();

    QString data = clipboard->text();
}
