/*****************************************************************************
 *            Created: 10/05/2012
 *          Copyright: (C) Copyright David Goodwin, 2012
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#ifndef TEMPLATEBROWSER_H
#define TEMPLATEBROWSER_H

#include <QDialog>

class QTreeWidgetItem;

namespace Ui {
class TemplateBrowser;
}

/** A window that allows the user to select a template. When an instance
 * is created the templates subdirectory is automatically scanned
 * and any .zgt are loaded into the list of templates. When a template is
 * selected a preview is rendered and the templates description is shown.
 */
class TemplateBrowser : public QDialog
{
    Q_OBJECT
    
public:
    explicit TemplateBrowser(QWidget *parent = 0);
    ~TemplateBrowser();

    /** Gets the filename for the currently selected template (if any)
     * @return The full filename for what ever template is currently selected.
     */
    QString currentTemplate() { return currentFile; }
    
public slots:
    /** Loads a .zgt file, renders a preview pixmap and displays its
     * description.
     * @param filename Name of the file to preview.
     */
    void previewFile(QString filename);

signals:
    /** Emitted when a template is selected.
     * @param filename The template that was selected.
     */
    void templateSelected(QString filename);

protected:
    void changeEvent(QEvent *e);
    
protected slots:
    void treeItemClicked(QTreeWidgetItem * item, int column);

    /** Lets clients know a template has been selected.
     */
    void place();

private:
    Ui::TemplateBrowser *ui; /*!< UI objects */

    /** Loads *.zgt files from the templates subdirectory and populates the
     * tree widget.
     */
    void populateTemplates();

    QString currentFile; /*!< The currently selected template (if any) */
};

#endif // TEMPLATEBROWSER_H
